//
//  TankModel.swift
//  Riego Automatico
//
//  Created by Santiago Pavón Gómez on 15/9/17.
//  Copyright © 2017 UPM. All rights reserved.
//

import Foundation

class TankModel {
    
    // Parámetro de entrada: Radio del deposito que contiene el agua para regar
    let radiusTank = 0.3    // 0.3 m == 30 cm
    
    // Parámetro de entrada: radio de la tuberia por donde sale el agua.
    let radiousPipe = 0.0025  // 0.0025 m == 2.5 mm
    
    // Parámetro de entrada: Altura inicial de agua en el deposito:
    let initialWaterHeight = 0.5    // 0.5 m == 50 cm
    
    // Valor auxiliar: Area o seccion del deposito.
    private let areaTank: Double
    
    // Valor auxiliar: Area o seccion de la tuberia.
    private let areaPipe: Double
    
    // Valor auxiliar muy usado: areaTank^2 / areaPipe^2 - 1.
    private let at2ap2m1: Double
   
    // Valor auxiliar muy usado: areaPipe^2 / areaTank^2 - 1.
    private let ap2at2m1: Double
    
    // Tiempo en vaciarse
    let timeToEmpty: Double
    
    // Inicializador
    init() {
      
        // Valores auxiliares
        areaTank = Double.pi * pow(radiusTank,2)
        areaPipe = Double.pi * pow(radiousPipe,2)
        at2ap2m1 = pow(areaTank/areaPipe,2)-1
        ap2at2m1 = pow(areaPipe/areaTank,2)-1
        
        // print("Area del tanque = \(areaTank)")
        // print("Area de la tuberia = \(areaPipe)")
        
        timeToEmpty = sqrt(2 * initialWaterHeight * at2ap2m1 / Constants.g)
    }
    
    // Velocidad de salida del agua en funcion de la altura
    func waterOutputSpeed(waterHeight h: Double) -> Double {
        let v = sqrt(-2 * Constants.g * h / ap2at2m1)
        
        return max(0, v)
    }
    
    // Velocidad de bajada del nivel de agua en el deposito en funcion de la altura
    func waterHeightSpeed(waterHeight h: Double) -> Double {
        let v = sqrt(2 * Constants.g * h / at2ap2m1)
        return max(0, v)
    }
    
    // Nivel/Altura del agua en el deposito en funcion del tiempo
    func waterHeightAt(time t: Double) -> Double {
        
        if t > timeToEmpty { return 0 }
        
        let c0 = initialWaterHeight
        let c1 = -sqrt(2 * Constants.g * initialWaterHeight / at2ap2m1)
        
        if c1.isNaN {
            return 0
        }
        
        let c2 = Constants.g / 2 / at2ap2m1
        
        return c0 + c1*t + c2*t*t
    }
}
